\ Group.ex

\ Example problem for use with LOGICA.
\ Copyright  1992, LogicLab.  All rights reserved.

\ This is a problem in group theory:
\ Show that a group in which the square of every element is the 
\ identity is a commutative group.
\
\ This is a solution to the problem presented on pp.243-58 of 
\ Wos, Overbeek, Lusk & Boyle, "Automated Reasoning", Prentice-Hall, Inc., 
\ Englewood Cliffs, NJ 07632 (1984).
\
Logic: paramod;
function: f,inv;    \ the group operation; inverse
constant: e,a,b;    \ e = the identity element

Demodulate:
rewrite(f(x,e),x);  \ definition of e
rewrite(f(e,x),x);  \ definition of e

Axiom: 
equate(f(f(x,x1),x2),f(x,f(x1,x2))); \ associativity, a property of groups
equate(f(x,inv(x)),e);               \ definition of inverse
equate(f(inv(x),x),e);               \ ditto

Assert:
equate(f(x,x),e);          \ Assume that the square of any element is e
~equate(f(a,b),f(b,a));    \ Claim that for some a, b, f(a,b) =/= f(b,a)

\ junk results
Trivial: equate(x,x);

\ parameters
LimitF: 2; 	      \ limits number of function symbols
LimitV: 2; 	      \ limits number of distinct variable symbols
LimitC: 0; 	      \ limits number of constant symbols


