\ Mockingbird.ex

\ Example problem for use with LOGICA.
\ Copyright  1992, LogicLab.  All rights reserved.

\ This problem is adapted from
\ R. Smullyan, "To Mock a Mockingbird",
\ Alfred A. Knopf, New York, 1985, p.74.

\ Smullyan imagines a forest in which the birds' songs are the names
\ of birds!  When bird A hears the name of bird B, bird A sings out
\ the name of some bird - it could be the name of A, or of B, or of 
\ a third bird.  A's response upon hearing "B" is written AB.  

\ Through this device, Smullyan introduces combinatory logic.  
\ Postulating the existence of birds with certain peculiar 
\ characteristics, he proceeds to develop a number of interesting 
\ results.

\ The mockingbird M is a bird with the property that when M 
\ hears the name of any bird X, he responds the way X would upon
\ hearing his own name: MX = XX. 
\ Bird A is said to be "fond" of bird B if A responds to "B" by 
\ singing "B", that is, if AB = B.
\ Let us assume that
\ 1) a mockingbird M exists, and  
\ 2) for every pair of birds X, Y there 
\ exists some bird, which we'll call comp(X,Y) which "composes" 
\ birds X and Y; that is, for all birds X,Y,Z, we have
\            X(YZ) = (comp(X,Y))Z.  
\ Can we prove that every bird must then be fond of some bird?  
\ This is the problem represented here.

\ To adapt this system to clause notation, let's denote the bird
\ response function by the function "says".  That is, instead of
\ writing AB to denote A's response upon hearing "B", we'll
\ write "says(A,B)".  Then the definition of the mockingbird M is
\            says(M,X) = says(X,X).
\ The composition rule is expressed as
\        says(comp(x,x1),x2) = says(x,(says(x1,x2))
\ We assert that there is some bird g which is not fond of any bird,
\             says(g,x) =/= x for all x
\ and then demonstrate that this leads to a contradiction; hence,
\ every bird must be fond of at least one bird.
\
Logic: paramod;
function: says;          \ says = the bird's answer function; 
function: comp;          \ comp = composition
constant: M,g; 

Axiom:    			\ for each pair of birds x, x1 there is a composite bird
Equate(says(comp(x,x1),x2),says(x,says(x1,x2)));

Assert:
Equate(says(M,x),says(x,x));   \ M is a mockingbird
~Equate(says(g,x),x);          \ bird "g" is not fond of any bird

\ junk results
Trivial: equate(x,x);

\ We'll find that g must be fond of V = says(comp(g,M),comp(g,M)):
\ says(g,says(M,comp(g,M)))
\                 = says(g,says(comp(g,M),comp(g,M)))    (def. of M)
\                 = says(comp(g,M),comp(g,M))            (def. of comp)
\ That is, says(g,V) = V.

\ LOAD and RUN this file.  The existence of a bird g which is
\ fond of no bird will be shown to be inconsistent with the other 
\ clauses of the system.  This is indicated by the "Unit Conflict"
\ result.  The identity of the bird of which g must be fond is
\ shown in the substitution given below the clauses found in conflict.

