\ Socrates.ex

\ Example problem for use with LOGICA.
\ Copyright  1992, LogicLab.  All rights reserved.

\ This file shows how to represent the classical syllogism:
\ "All men are mortal; Socrates is a man; therefore, Socrates is mortal".
\ This example illustrates the basic syntactical elements which COGITO 
\ understands, the principle of unification, and the inference rule 
\ "unit-result resolution".

\ Syntactical elements illustrated here are: declarations, constants 
\ and logical variables, atoms and clauses, and lists of clauses.

\ The inference rule for this program is to be unit-result resolution:

Logic: URres;

\ The sybols "Man", "Woman" and "Mortal" are to be understood as 
\ Predicates, that is, functions which take the values "true" and "false".

Predicate: Man, Woman, Mortal;

\ "Socrates" and "JaneAusten" are to be understood as Constant symbols.

Constant: Socrates, JaneAusten;

\ The next line specifies that subsequent clauses should be placed 
\ in the "Axiom" list.
 
Axiom: 

\ The next two lines specify clauses to be placed in the Axiom list.

~Man(x) | Mortal(x);
~Man(x) | ~Woman(x);


\ The subsequent clauses will be placed in the "Assert" list.

Assert:
Man(Socrates);
Woman(JaneAusten);
~Mortal(JaneAusten);

\ Use the "LOAD" command in the RUN menu to compile this program.  
\ Then use the "GO" command in the RUN menu to obtain results.
\
\ Unit-result resolution uses a non-unit clause as "nucleus" and 
\ attempts to unify complementary unit "satellite" clauses with all but 
\ one of the nucleus's atoms.  For example, using the nucleus
\
\ ~Man(x) | Mortal(x)
\
\ and the unit clause 
\
\ Man(Socrates),
\
\ unification (pattern-matching) of "~Man(x)" with "Man(Socrates)" 
\ results in the variable-binding {x := Socrates}.  This binding is then
\ applied to the unused atom in the nucleus, to yield the deduction
\
\ Mortal(Socrates).
\
\ The other results from this example can be understood similarly. 
\
\ 

